@@ -261,22 +261,51 @@ icon_theme_changed_cb(GtkIconTheme *icon_theme, gpointer user_data)
 static char* 
 bytes_to_string(double bytes, gboolean per_sec, gboolean bits)
 {
-	if (bits)
+	const char *format;
+	const char *unit;
+	guint kilo; /* no really a kilo : a kilo or kibi */
+
+	if (bits) {
 		bytes *= 8;
-	
-	if (bytes < 1000)
-		return g_strdup_printf("%.0f %s", bytes, 
-						per_sec ? bits ? _("b/s") : _("B/s"): _("byte"));
-	if (bytes < 100000)
-		return g_strdup_printf("%.1f %s", bytes / 1024, 
-						per_sec ? bits ? _("kb/s") :_("kB/s") : _("kbyte"));
-	if (bytes < 1000000)
-		return g_strdup_printf("%.0f %s", bytes / 1024, 
-						per_sec ? bits ? _("kb/s") : _("kB/s") : _("kbyte"));
-	return g_strdup_printf("%.1f %s", bytes / 1024 / 1024, 
-						per_sec ? bits ? _("Mb/s") : _("MB/s") : _("Mbyte"));
+		kilo = 1000;
+	} else
+		kilo = 1024;
+
+	if (bytes < kilo) {
+
+		format = "%.0f %s";
+
+		if (per_sec)
+			unit = bits ? N_("b/s")   : N_("B/s");
+		else
+			unit = bits ? N_("bits")  : N_("bytes");
+
+	} else if (bytes < (kilo * kilo)) {
+		format = (bytes < (100 * kilo)) ? "%.1f %s" : "%.0f %s";
+		bytes /= kilo;
+
+		if (per_sec)
+			unit = bits ? N_("kb/s") : N_("KiB/s");
+		else
+			unit = bits ? N_("kb")   : N_("KiB");
+
+	} else {
+
+		format = "%.1f %s";
+
+		bytes /= kilo * kilo;
+
+		if (per_sec)
+			unit = bits ? N_("Mb/s") : N_("MiB/s");
+		else
+			unit = bits ? N_("Mb")   : N_("MiB");
+	}
+
+
+	return g_strdup_printf(format, bytes, gettext(unit));
 }
 
+
 /* Redraws the graph drawingarea
  * Some really black magic is going on in here ;-)
  */
@@ -380,20 +409,21 @@ search_for_up_if(NetspeedApplet *applet)
 	default_route = get_default_route();
     
 	if (default_route != NULL) {
-		info = get_device_info(default_route);
+		get_device_info(default_route, &info);
 		if (info.running) {
 			free_device_info(&applet->devinfo);
 			applet->devinfo = info;
 			applet->device_has_changed = TRUE;
 			return;
 		}
+		free_device_info(&info);
 	}
 
 	devices = get_available_devices();
 	for (tmp = devices; tmp; tmp = g_list_next(tmp)) {
-		if (strcmp(tmp->data, "lo") == 0) continue;
-		if (strncmp(tmp->data, "dummy", strlen("dummy")) == 0) continue;
-		info = get_device_info(tmp->data);
+		if (is_no_dummy_device(tmp->data) == FALSE)
+			continue;
+		get_device_info(tmp->data, &info);
 		if (info.running) {
 			free_device_info(&applet->devinfo);
 			applet->devinfo = info;
@@ -405,10 +435,6 @@ search_for_up_if(NetspeedApplet *applet)
 	free_devices_list(devices);
 }
 
-#ifndef max
-#define max(a, b) (((a) > (b)) ? (a) : (b))
-#endif
-
 /* Here happens the really interesting stuff */
 static void
 update_applet(NetspeedApplet *applet)
@@ -424,7 +450,7 @@ update_applet(NetspeedApplet *applet)
 	
 /* First we try to figure out if the device has changed */
 	oldinfo = applet->devinfo;
-	applet->devinfo = get_device_info(oldinfo.name);
+	get_device_info(oldinfo.name, &applet->devinfo);
 	if (compare_device_info(&applet->devinfo, &oldinfo))
 		applet->device_has_changed = TRUE;
 	free_device_info(&oldinfo);
@@ -450,10 +476,10 @@ update_applet(NetspeedApplet *applet)
 /* create the strings for the labels and tooltips */
 	if (applet->devinfo.running)
 	{	
-		if (applet->devinfo.rx  < applet->in_old[OLD_VALUES - 1]) indiff = 0;
-		else indiff = applet->devinfo.rx - applet->in_old[OLD_VALUES - 1];
-		if (applet->devinfo.tx  < applet->out_old[OLD_VALUES - 1]) outdiff = 0;
-		else outdiff = applet->devinfo.tx - applet->out_old[OLD_VALUES - 1];
+		if (applet->devinfo.rx < applet->in_old[applet->index_old]) indiff = 0;
+		else indiff = applet->devinfo.rx - applet->in_old[applet->index_old];
+		if (applet->devinfo.tx < applet->out_old[applet->index_old]) outdiff = 0;
+		else outdiff = applet->devinfo.tx - applet->out_old[applet->index_old];
 		
 		inrate = indiff * 1000.0;
 		inrate /= (double)(applet->refresh_time * OLD_VALUES);
@@ -462,23 +488,28 @@ update_applet(NetspeedApplet *applet)
 		
 		applet->in_graph[applet->index_graph] = inrate;
 		applet->out_graph[applet->index_graph] = outrate;
-		applet->max_graph = max(inrate, applet->max_graph);
-		applet->max_graph = max(outrate, applet->max_graph);
+		applet->max_graph = MAX(inrate, applet->max_graph);
+		applet->max_graph = MAX(outrate, applet->max_graph);
 		
 		in = bytes_to_string(inrate, TRUE, applet->show_bits);
 		out = bytes_to_string(outrate, TRUE, applet->show_bits);
 		sum = bytes_to_string(inrate + outrate, TRUE, applet->show_bits);
 		
-		if (applet->show_sum) {
-			tooltip = g_strdup_printf(_("%s :%s\nin: %s out: %s"), 
-				applet->devinfo.name, 
-				applet->devinfo.ip ? applet->devinfo.ip : _("has no ip"),
-				in, out);
-		} else {
-			tooltip = g_strdup_printf(_("%s: %s\nsum: %s"), 
-				applet->devinfo.name, 
-				applet->devinfo.ip ? applet->devinfo.ip : _("has no ip"), 
-				sum);
+
+		if (applet->show_tooltip) {
+			if (applet->show_sum) {
+				tooltip = g_strdup_printf(
+					_("%s: %s\nin: %s out: %s"),
+					applet->devinfo.name,
+					applet->devinfo.ip ? applet->devinfo.ip : _("has no ip"),
+					in, out);
+			} else {
+				tooltip = g_strdup_printf(
+					_("%s: %s\nsum: %s"),
+					applet->devinfo.name,
+					applet->devinfo.ip ? applet->devinfo.ip : _("has no ip"),
+					sum);
+			}
 		}
 	} else {
 		in = g_strdup("");
@@ -486,30 +517,39 @@ update_applet(NetspeedApplet *applet)
 		sum = g_strdup("");
 		applet->in_graph[applet->index_graph] = 0;
 		applet->out_graph[applet->index_graph] = 0;
-		tooltip = g_strdup_printf(_("%s is down"), applet->devinfo.name);
+		if (applet->show_tooltip)
+			tooltip = g_strdup_printf(_("%s is down"), applet->devinfo.name);
 	}
 		
 /* Refresh the text of the labels and tooltip */
-	add_markup_size(&sum, applet->font_size);
-	add_markup_size(&in, applet->font_size);
-	add_markup_size(&out, applet->font_size);
-	gtk_label_set_markup(GTK_LABEL(applet->sum_label), sum);
-	gtk_label_set_markup(GTK_LABEL(applet->in_label), in);
-	gtk_label_set_markup(GTK_LABEL(applet->out_label), out);
-	gtk_tooltips_set_tip(applet->tooltips, GTK_WIDGET(applet->applet), tooltip, "");
+	if (applet->show_sum) {
+		add_markup_size(&sum, applet->font_size);
+		gtk_label_set_markup(GTK_LABEL(applet->sum_label), sum);
+	} else {
+		add_markup_size(&in, applet->font_size);
+		add_markup_size(&out, applet->font_size);
+		gtk_label_set_markup(GTK_LABEL(applet->in_label), in);
+		gtk_label_set_markup(GTK_LABEL(applet->out_label), out);
+	}
+
 	g_free(in);
 	g_free(out);
 	g_free(sum);
-	g_free(tooltip);
+
+	if (applet->show_tooltip) {
+		g_debug("updating tooltip");
+		gtk_tooltips_set_tip(applet->tooltips, GTK_WIDGET(applet->applet), tooltip, "");
+		g_free(tooltip);
+	}
 
 /* Refresh the values of the Infodialog */
 	if (applet->inbytes_text) {
-		inbytes = bytes_to_string((double)applet->devinfo.rx, FALSE, FALSE);
+		inbytes = bytes_to_string((double)applet->devinfo.rx, FALSE, applet->show_bits);
 		gtk_label_set_text(GTK_LABEL(applet->inbytes_text), inbytes);
 		g_free(inbytes);
 	}	
 	if (applet->outbytes_text) {
-		outbytes = bytes_to_string((double)applet->devinfo.tx, FALSE, FALSE);
+		outbytes = bytes_to_string((double)applet->devinfo.tx, FALSE, applet->show_bits);
 		gtk_label_set_text(GTK_LABEL(applet->outbytes_text), outbytes);
 		g_free(outbytes);
 	}
@@ -518,13 +558,9 @@ update_applet(NetspeedApplet *applet)
 		redraw_graph(applet);
 	
 /* Save old values... */
-	for (i = OLD_VALUES - 1; i > 0; i--)
-	{
-		applet->in_old[i] = applet->in_old[i - 1];
-		applet->out_old[i] = applet->out_old[i - 1];
-	}
-	applet->in_old[0] = applet->devinfo.rx;
-	applet->out_old[0] = applet->devinfo.tx;
+	applet->in_old[applet->index_old] = applet->devinfo.rx;
+	applet->out_old[applet->index_old] = applet->devinfo.tx;
+	applet->index_old = (applet->index_old + 1) % OLD_VALUES;
 
 /* Move the graphindex. Check if we can scale down again */
 	applet->index_graph = (applet->index_graph + 1) % GRAPH_VALUES; 
@@ -533,8 +569,8 @@ update_applet(NetspeedApplet *applet)
 		double max = 0;
 		for (i = 0; i < GRAPH_VALUES; i++)
 		{
-			max = max(max, applet->in_graph[i]);
-			max = max(max, applet->out_graph[i]);
+			max = MAX(max, applet->in_graph[i]);
+			max = MAX(max, applet->out_graph[i]);
 		}
 		applet->max_graph = max;
 	}
@@ -601,17 +637,13 @@ about_cb(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 		"Jörgen Scheibengruber <mfcn@gmx.de>", 
 		"Dennis Cranston <dennis_cranston@yahoo.com>",
 		"Pedro Villavicencio Garrido <pvillavi@gnome.org>",
-		"Benoît Dejean <TazForEver@dlfp.org>",
+		"Benoît Dejean <benoit@placenet.org>",
 		NULL
 	};
     
 	const char *website = "http://www.wh-hms.uni-ulm.de/~mfcn/netspeed/";
 	const char *website_label = _("Netspeed Website");
         
-
-	/* Feel free to put your names here translators :-) */
-	char *translators = _("TRANSLATORS");
-	
 	gtk_about_dialog_set_email_hook ((GtkAboutDialogActivateLinkFunc) handle_links,
 					 GINT_TO_POINTER (LINK_TYPE_EMAIL), NULL);
 	
@@ -625,7 +657,7 @@ about_cb(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 			       "comments", _("A little applet that displays some information on the traffic on the specified network device"),
 			       "authors", authors, 
 			       "documenters", NULL, 
-			       "translator-credits", strcmp ("TRANSLATORS", translators) ? translators : NULL, 
+			       "translator-credits", _("translator-credits"),
 			       "website", website,
 			       "website-label", website_label,
 			       "logo-icon-name", LOGO_ICON,
@@ -658,7 +690,7 @@ device_change_cb(GtkComboBox *combo, NetspeedApplet *applet)
 	if (g_str_equal(ptr->data, applet->devinfo.name))
 		return;
 	free_device_info(&applet->devinfo);
-	applet->devinfo = get_device_info(ptr->data);
+	get_device_info(ptr->data, &applet->devinfo);
 	applet->device_has_changed = TRUE;
 	update_applet(applet);
 }
@@ -939,7 +971,7 @@ settings_cb(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_sum_checkbutton), applet->show_sum);
 	gtk_box_pack_start(GTK_BOX(controls_vbox), show_sum_checkbutton, FALSE, FALSE, 0);
 	
-	show_bits_checkbutton = gtk_check_button_new_with_mnemonic(_("Show _bits/s (b/s) instead of bytes/s (B/s)"));
+	show_bits_checkbutton = gtk_check_button_new_with_mnemonic(_("Show _bits instead of bytes"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_bits_checkbutton), applet->show_bits);
 	gtk_box_pack_start(GTK_BOX(controls_vbox), show_bits_checkbutton, FALSE, FALSE, 0);
 	
@@ -1331,6 +1363,33 @@ applet_destroy(PanelApplet *applet_widget, NetspeedApplet *applet)
 	return;
 }
 
+
+static gboolean
+netspeed_enter_cb(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
+{
+	NetspeedApplet *applet = data;
+
+	g_debug("enter");
+	applet->show_tooltip = TRUE;
+	update_applet(applet);
+
+	return TRUE;
+}
+
+
+static gboolean
+netspeed_leave_cb(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
+{
+	NetspeedApplet *applet = data;
+
+	g_debug("leave");
+	applet->show_tooltip = FALSE;
+	return TRUE;
+}
+
+
+
+
 /* The "main" function of the applet
  */
 static gboolean
@@ -1344,6 +1403,8 @@ netspeed_applet_factory(PanelApplet *applet_widget, const gchar *iid, gpointer d
 	if (strcmp (iid, "OAFIID:GNOME_NetspeedApplet"))
 		return FALSE;
 
+    glibtop_init();
+
     icon_theme = gtk_icon_theme_get_default();
     gtk_icon_theme_append_search_path(icon_theme, DATADIR"/pixmaps/"PACKAGE);
 	
@@ -1395,7 +1456,7 @@ netspeed_applet_factory(PanelApplet *applet_widget, const gchar *iid, gpointer d
 		tmp = panel_applet_gconf_get_string(applet_widget, "device", NULL);
 		if (tmp && strcmp(tmp, "")) 
 		{
-			applet->devinfo = get_device_info(tmp);
+			get_device_info(tmp, &applet->devinfo);
 			g_free(tmp);
 		}
 		tmp = panel_applet_gconf_get_string(applet_widget, "up_command", NULL);
@@ -1428,17 +1489,20 @@ netspeed_applet_factory(PanelApplet *applet_widget, const gchar *iid, gpointer d
 	if (applet->font_size < 1)
 		applet->font_size = get_default_font_size();
 	
-	if (!applet->devinfo.name)  {
+	if (!applet->devinfo.name) {
 		GList *ptr, *devices = get_available_devices();
 		ptr = devices;
 		while (ptr) { 
-			if (!g_str_equal(ptr->data, "lo"))
-				applet->devinfo = get_device_info(ptr->data);
+			if (!g_str_equal(ptr->data, "lo")) {
+				get_device_info(ptr->data, &applet->devinfo);
+				break;
+			}
 			ptr = g_list_next(ptr);
 		}
 		free_devices_list(devices);		
 	}
-	if (!applet->devinfo.name) applet->devinfo = get_device_info("lo");	
+	if (!applet->devinfo.name)
+		get_device_info("lo", &applet->devinfo);	
 	applet->device_has_changed = TRUE;	
 	
 	applet->tooltips = gtk_tooltips_new();
@@ -1497,6 +1561,15 @@ netspeed_applet_factory(PanelApplet *applet_widget, const gchar *iid, gpointer d
                            G_CALLBACK(applet_button_press),
                            (gpointer)applet);
 
+	g_signal_connect(G_OBJECT(applet_widget), "leave_notify_event",
+			 G_CALLBACK(netspeed_leave_cb),
+			 (gpointer)applet);
+
+	g_signal_connect(G_OBJECT(applet_widget), "enter_notify_event",
+			 G_CALLBACK(netspeed_enter_cb),
+			 (gpointer)applet);
+
+
 	menu_string = g_strdup_printf(netspeed_applet_menu_xml, _("Device _Details"), _("_Preferences..."), _("_Help"), _("_About..."));
 	panel_applet_setup_menu(applet_widget, menu_string,
                                  netspeed_applet_menu_verbs,
